using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Security.Instrumentation;
using System.Collections.Specialized;
using System.Security.Principal;
using System.Security;
using System.Diagnostics;
namespace Kreeg.EnterpriseLibrary.Security.Database.Authorization
{
    [ConfigurationElementType(typeof(CustomAuthorizationProviderData))]
    public class DbRulesAuthorizationProvider : AuthorizationProvider 
    {

        private IDictionary<string, IAuthorizationRule> authorizationRules;


        string databaseInstance = "";
        string getRuleByNameStoredProcName = "dbo.aspnet_Rule_GetRuleByName";
        string getAllRulesStoredProcName = "dbo.aspnet_Rule_GetAllRules";
        string insertRuleNameStoredProcName = "dbo.aspnet_Rule_InsertRule";
        string updateRuleByNameStoredProcName = "dbo.aspnet_Rule_UpdateRuleByName";
        string deleteRuleByNameStoredProcName = "dbo.aspnet_Rule_DeleteRuleByName";
        private Microsoft.Practices.EnterpriseLibrary.Data.Database database = null;
        private DbRulesManager mgr = null;
        public DbRulesAuthorizationProvider(NameValueCollection configurationItems)
        {
            databaseInstance = configurationItems["DatabaseInstance"];
            //getRuleByNameStoredProcName = configurationItems["GetRuleByNameStoredProcName"];
            //getAllRulesStoredProcName = configurationItems["GetAllRulesStoredProcName"];
            //insertRuleNameStoredProcName = configurationItems["InsertRuleNameStoredProcName"];
            //updateRuleByNameStoredProcName = configurationItems["UpdateRuleByNameStoredProcName"];
            //deleteRuleByNameStoredProcName = configurationItems["DeleteRuleByNameStoredProcName"];
            database = Microsoft.Practices.EnterpriseLibrary.Data.DatabaseFactory.CreateDatabase(databaseInstance);

            if (mgr == null)
            {
                mgr = new DbRulesManager(database, getAllRulesStoredProcName, getAllRulesStoredProcName, insertRuleNameStoredProcName, updateRuleByNameStoredProcName, deleteRuleByNameStoredProcName);
            }
        }
        public DbRulesAuthorizationProvider()
        {
            database = Microsoft.Practices.EnterpriseLibrary.Data.DatabaseFactory.CreateDatabase();

            if (mgr == null)
            {
                mgr = new DbRulesManager(database, getAllRulesStoredProcName, getAllRulesStoredProcName, insertRuleNameStoredProcName, updateRuleByNameStoredProcName, deleteRuleByNameStoredProcName);
            }
        }
        //public override void DeleteRuleByName(string Name)
        //{
        //    mgr.DeleteRuleByName(Name);
        //}
        //public override void UpdateRuleByName(string Name, string expression)
        //{
        //    mgr.UpdateRuleByName(Name, expression);
        //}
        //public override void InsertRule(string Name, string expression)
        //{
        //    mgr.InsertRule(Name, expression);
        //}
        //public override NamedElementCollection<AuthorizationRuleData> GetAllRulesAsCollection()
        //{
        //   return mgr.GetAllRulesAsCollection();
        //}
        //public override System.Data.DataSet GetAllRules()
        //{
        //    return mgr.GetAllRules();
        //}
        //public override AuthorizationRuleData GetRule(string Name)
        //{
        //    return mgr.GetRule( Name);
        //}
        public override bool Authorize(IPrincipal principal, string context)
        {
            if (principal == null)
            {
                throw new ArgumentNullException("principal");
            }
            if (context == null || context.Length == 0)
            {
                throw new ArgumentNullException("context");
            }

           InstrumentationProvider.FireAuthorizationCheckPerformed(principal.Identity.Name, context);

            BooleanExpression expression = GetParsedExpression(context, mgr);
            if (expression == null)
            {
                throw new SecurityException(String.Format("Authorization Rule {0} not found in the database.", context));
            }

            bool result = expression.Evaluate(principal);

            if (result == false)
            {
                InstrumentationProvider.FireAuthorizationCheckFailed(principal.Identity.Name, context);
            }
            return result;
        }
        /// <summary>
        /// Retrieves a rule from the database and parses it into a boolean expresssion
        /// </summary>
        /// <param name="context">The Rule name</param>
        /// <param name="mgr">The Database Rules Manager that queries the database</param>
        /// <returns>A BooleanExpression object</returns>
        private BooleanExpression GetParsedExpression(string context, DbRulesManager mgr)
        {

            AuthorizationRuleData rule = mgr.GetRule(context);
            if (rule == null)
            {
                return null;
            }

            Parser p = new Parser();
            return p.Parse(rule.Expression);

        }
        private BooleanExpression GetParsedExpression(string ruleName)
        {
            IAuthorizationRule rule = null;
            authorizationRules.TryGetValue(ruleName, out rule);
            if (rule == null) return null;

            string expression = rule.Expression;
            Parser parser = new Parser();

            return parser.Parse(expression);
        }

    }

}
