IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[aspnet_AuthorizationRules]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
begin
  drop table aspnet_AuthorizationRules
end

go

CREATE TABLE [dbo].[aspnet_AuthorizationRules] (
 [RuleID] [int] IDENTITY (1, 1) NOT NULL ,
 [Name] [varchar] (255) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL ,
 [Expression] [ntext] COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL DEFAULT 'NOT I:?' ,
 [Describe] [ntext] COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL DEFAULT '' 

) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO
ALTER TABLE [dbo].[aspnet_AuthorizationRules] ADD 
 CONSTRAINT [PK_aspnet_AuthorizationRules] PRIMARY KEY  CLUSTERED 
 (
  [RuleID]
 )  ON [PRIMARY] 
GO

TRUNCATE TABLE [dbo].[aspnet_AuthorizationRules]

DBCC CHECKIDENT ('[dbo].[aspnet_AuthorizationRules]', RESEED, 1)

SET IDENTITY_INSERT [dbo].[aspnet_AuthorizationRules] ON

INSERT INTO [dbo].[aspnet_AuthorizationRules] (	[RuleID],	[Name],	[Expression])
	SELECT 1, 'Hire Employee', 'R:Manager OR R:HumanResources'

INSERT INTO [dbo].[aspnet_AuthorizationRules] (	[RuleID],	[Name],	[Expression])
	SELECT 2, 'Print Document', 'NOT I:?' 

INSERT INTO [dbo].[aspnet_AuthorizationRules] (	[RuleID],	[Name],	[Expression])
	SELECT 3, 'Submit New Order', '(R:Employee AND (NOT I:Bob))'
	
SET IDENTITY_INSERT [dbo].[aspnet_AuthorizationRules] OFF



if exists (select * from sysobjects where id = object_id(N'[dbo].[aspnet_Rule_GetRuleByName]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[aspnet_Rule_GetRuleByName]
GO

CREATE PROCEDURE dbo.aspnet_Rule_GetRuleByName
	@name nvarchar(255)
/*************************************************************************************************************
* Purpose:	Retrieves a rule
*
* Date		By		Description
* ----------	---------------	-------------------------------------------------------------------
  03/01/2005	Craig Vermeer	Retrieves a Rule from the database

************************************************************************************************************/
AS

	SELECT RuleId, Name, Expression,Describe
	FROM aspnet_AuthorizationRules
	WHERE Name = @name

RETURN(0)


GO
if exists (select * from sysobjects where id = object_id(N'[dbo].[aspnet_Rule_InsertRule]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[aspnet_Rule_InsertRule]
GO

CREATE PROCEDURE dbo.aspnet_Rule_InsertRule
	@name nvarchar(255),
	@expression nvarchar(2000),
	@Describe nvarchar(2000)
/*************************************************************************************************************
* Purpose:	Inserts a Rule
*
* Date		By		Description
* ----------	---------------	-------------------------------------------------------------------
  05/29/2005	Craig Vermeer	Inserts a Rule

************************************************************************************************************/
AS
BEGIN
	INSERT INTO dbo.aspnet_AuthorizationRules
	(name, expression ,Describe)
	VALUES(@name, @expression,@Describe)
	
END
GO


if exists (select * from sysobjects where id = object_id(N'[dbo].[aspnet_Rule_GetAllRules]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[aspnet_Rule_GetAllRules]
GO

CREATE PROCEDURE dbo.aspnet_Rule_GetAllRules
/*************************************************************************************************************
* Purpose:	Retrieves all rules
*
* Date		By		Description
* ---------	---------------	-------------------------------------------------------------------
  05/29/2005	Craig Vermeer	Retrieves a Rule from the database

************************************************************************************************************/
AS

	SELECT RuleId, Name, Expression
	FROM aspnet_AuthorizationRules

RETURN(0)


GO

if exists (select * from sysobjects where id = object_id(N'[dbo].[aspnet_Rule_UpdateRuleByName]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[aspnet_Rule_UpdateRuleByName]
GO

CREATE PROCEDURE dbo.aspnet_Rule_UpdateRuleByName
	@name nvarchar(255),
	@expression nvarchar(2000),
	@Describe nvarchar(2000)
/*************************************************************************************************************
* Purpose:	Updates a Rule
*
* Date		By		Description
* ----------	---------------	-------------------------------------------------------------------
  05/29/2005	Craig Vermeer	Updates a Rule

************************************************************************************************************/
AS
BEGIN

	UPDATE dbo.aspnet_AuthorizationRules
	SET expression = @expression ,Describe=@Describe
	WHERE [name] = @name

END
GO



if exists (select * from sysobjects where id = object_id(N'[dbo].[aspnet_Rule_DeleteRuleByName]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[aspnet_Rule_DeleteRuleByName]
GO

CREATE PROCEDURE dbo.aspnet_Rule_DeleteRuleByName
	@name nvarchar(255)
/*************************************************************************************************************
* Purpose:	Deletes a rule
*
* Date		By		Description
* ----------	---------------	-------------------------------------------------------------------
  05/29/2005	Craig Vermeer	Deletes a Rule

************************************************************************************************************/
AS
BEGIN

	DELETE dbo.aspnet_AuthorizationRules
	WHERE [name] = @name

END
GO


