
using System;
using System.ComponentModel;
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration.Design;
using Kreeg.EnterpriseLibrary.Security.Database.Authorization;
using System.Drawing;
using System.Drawing.Design;

namespace Kreeg.EnterpriseLibrary.Security.Database.Authorization.Configuration
{
    /// <summary>
    /// Node representing configuration from <see cref="DbRulesAuthorizationProviderData"/>.
    /// </summary>
    public class DbRulesAuthorizationProviderNode : AuthorizationProviderNode
    {
        private string databaseName;
        private const string applicationName = "My Application";
        private ConnectionStringSettingsNode database;
        private EventHandler<ConfigurationNodeChangedEventArgs> onDatabaseRemoved;
        private EventHandler<ConfigurationNodeChangedEventArgs> onDatabaseRenamed;


        private DbRulesAuthorizationProviderData DbRulesAuthorizationProviderData;

        /// <summary>
        /// Default constructor
        /// </summary>
        public DbRulesAuthorizationProviderNode()
            : this(new DbRulesAuthorizationProviderData("Database Authorization Provider", string.Empty))
        {
        }

        /// <summary>
        /// Constructor for DbRulesAuthorizationProviderDataNode.
        /// </summary>
        /// <param name="DbRulesAuthorizationProviderData">Configuration data for <see cref="DbRulesAuthorizationProvider"></see></param>
        public DbRulesAuthorizationProviderNode(DbRulesAuthorizationProviderData DbRulesAuthorizationProviderData)            
        {
            this.DbRulesAuthorizationProviderData = DbRulesAuthorizationProviderData;
            this.onDatabaseRemoved = new EventHandler<ConfigurationNodeChangedEventArgs>(this.OnDatabaseRemoved);
            this.onDatabaseRenamed = new EventHandler<ConfigurationNodeChangedEventArgs>(this.OnDatabaseRenamed);
        }

        /// <summary>
        /// Gets or sets the database instance that will be used to authenticate credentials.
        /// </summary>
        /// <value>An <see cref="InstanceNode"/> object.</value>
        [Editor(typeof(ReferenceEditor), typeof(UITypeEditor))]
        [ReferenceType(typeof(ConnectionStringSettingsNode))]
        [Required]
        [Description("The database instance that will be used to query for rules")]
        public ConnectionStringSettingsNode Database
        {
            get { return this.database; }
            
            set
            {
                database = LinkNodeHelper.CreateReference<ConnectionStringSettingsNode>(database,
																					value,
                                                                                    onDatabaseRemoved,
                                                                                    onDatabaseRenamed);

                databaseName = (database == null) ? String.Empty : database.Name;

            }
        }




        ///// <summary>
        ///// The fully qualified type name for the My provider 
        ///// </summary>
        //[Browsable(false)]
        //public override string TypeName
        //{
        //    get { return typeof(DbRulesAuthorizationProvider).AssemblyQualifiedName; }
        //}

        /// <summary>
        /// See <see cref="AuthorizationProviderNode.AuthorizationProviderData"/>.
        /// </summary>
        [Browsable(false)]
        public override AuthorizationProviderData AuthorizationProviderData
        {
            get { return new DbRulesAuthorizationProviderData(); }
        }

        ///// <summary>
        ///// Resolves the database node reference.
        ///// </summary>
        //public override void ResolveNodeReferences()
        //{
        //    base.ResolveNodeReferences();
        //    DatabaseSettingsNode databaseSettingsNode = Hierarchy.FindNodeByType(typeof(DatabaseSettingsNode)) as DatabaseSettingsNode;
        //    Debug.Assert(databaseSettingsNode != null, "How is it that the database settings are not there?");
        //    InstanceCollectionNode instanceCollectionNode = Hierarchy.FindNodeByType(typeof(InstanceCollectionNode)) as InstanceCollectionNode;
        //    this.Database = Hierarchy.FindNodeByName(instanceCollectionNode, this.DbRulesAuthorizationProviderData.Database) as InstanceNode;

        //}

        //protected override void AddDefaultChildNodes()
        //{
        //    base.AddDefaultChildNodes();
        //    CreateDatabaseSettingsNode();
        //}

        private void OnDatabaseRemoved(object sender, ConfigurationNodeChangedEventArgs args)
        {
            this.Database = null;
        }

        private void OnDatabaseRenamed(object sender, ConfigurationNodeChangedEventArgs args)
        {
            this.DbRulesAuthorizationProviderData.Database = args.Node.Name;
        }

        private void CreateDatabaseSettingsNode()
        {
            if (!DatabaseSettingsNodeExists())
            {
                AddChildNodeCommand cmd = new AddChildNodeCommand(Site, typeof(ConnectionStringSettingsNode));
                cmd.Execute(Hierarchy.RootNode);
            }
        }

        private bool DatabaseSettingsNodeExists()
        {
            ConnectionStringSettingsNode node = Hierarchy.FindNodeByType(typeof(ConnectionStringSettingsNode)) as ConnectionStringSettingsNode;
            return (node != null);
        }
    }
}